//
// Frequency & Phase Ditector for DigitalPhaseLockedLoop
//
// Copyright(c) 2002,2003 Tatsuyuki Satoh , All rights reserved
//
// Important !
//
// 1. License
//
// This program is freeware for non-commercial use. 
// The user of this source must assume all legal 
// responsibility in the use of this program. 
// This is not GPL.
//
// 2. Guarantee 
//
// An author does no guarantee about this program.
// You can use this under your own risk. 
// 
// 3. A rule to distribute.
//   Do either 1) or 2).
//   1)Include a perfect original source.
//   2)Include all these headers , and add document of 
//     modification points
//
// Histry
// 2003. 7.29 Rebuild with eddge sense logic.
// 2002.11.22 1st releaes
// 2002.11.18 Project Start
//

module pll(
//input
  clk_a,div_a,clk_b,div_b,
//output
  clkd_a,clkd_b,
  phase_a,phase_b,
  lock
);

/* configration */
parameter CNT_A_BITS = 14; // CLK_A divider bits
parameter CNT_B_BITS = 16; // CLK_B divider bits

parameter LOCK_A_CNT =  2; // CLK_A UnLock detect point
parameter LOCK_B_CNT =  2; // CLK_B UnLock ditect point

/* module port */
input  clk_a;                   // A Clock Input
input  clk_b;                   // B Clock Input
input  [CNT_A_BITS-1:0] div_a;  // A divide counter (unit = cycle-1)
input  [CNT_B_BITS-1:0] div_b;  // B divide counter (unit = cycle-1)
output clkd_a;                  // A Divided Clock Output
output clkd_b;                  // B Divided Clock Output
output phase_a;                 // A Phase Ditect (A>B,for slow loopback)
output phase_b;                 // B Phase Ditect (A<B,for slow loopback)
output lock;                    // Phase Lock (if freqency locked) *1

/*

*1: lock can't use tristate-controll for loopback output pin
    Because it is delayed from phase_a or phase_b signal.

*/

/////////////////////////////////////////////////////////////////////////////
// A/B Divider
/////////////////////////////////////////////////////////////////////////////

reg [CNT_A_BITS-1:0] cnt_a;
reg [CNT_B_BITS-1:0] cnt_b;
reg carry_a , carry_b;
reg unlock_a , unlock_b;

always @(posedge clk_a)
begin
  // Divider
  cnt_a   <= (cnt_a==div_a) ? 0 : cnt_a + 1;
  // Carry out
  carry_a <= (cnt_a==div_a);
  // lock ditect
  if(cnt_a==LOCK_A_CNT) unlock_a <= phase_a;
end

always @(posedge clk_b)
begin
  // Divider
  cnt_b   <= (cnt_b==div_b) ? 0 : cnt_b + 1;
  // Carry out
  carry_b <= (cnt_b==div_b);
  // lock ditect
  if(cnt_b==LOCK_B_CNT) unlock_b <= phase_b;
end

/////////////////////////////////////////////////////////////////////////////
// Phase Ditector
/////////////////////////////////////////////////////////////////////////////
reg  ph_a , ph_b;

wire ph_sync = ph_a & ph_b;

// Phase A
always @(posedge carry_a or posedge ph_sync)
  ph_a <= ph_sync ? 1'b0 :  1'b1;

// Phase B
always @(posedge carry_b or posedge ph_sync)
  ph_b <= ph_sync ? 1'b0 :  1'b1;

/////////////////////////////////////////////////////////////////////////////
// Lock Filter
/////////////////////////////////////////////////////////////////////////////
reg [3:0] lock_r;
wire unlock = unlock_a | unlock_b;
always @(posedge carry_a or posedge unlock)
  lock_r <= unlock ? 4'b0000 : {lock_r[3:0],1'b1};

/////////////////////////////////////////////////////////////////////////////
// OUTPUT I/F
/////////////////////////////////////////////////////////////////////////////
assign clkd_a  = carry_a;
assign clkd_b  = carry_b;
assign phase_a = ph_a;
assign phase_b = ph_b;
assign lock    = (lock_r==4'b1111);

endmodule

